"use strict";

let portResponses = { },
    portDisconnectFunc = null,
    port = null;

function createPort(nameArg, onDisconnect = null, ...funcs) {
    const funcNames = [ "onStartup" ];

    if(nameArg != PORT_C)
        funcNames.push("onBackgroundMessage");

    for(let i = 0; i < funcNames.length; i++)
        if(typeof window[funcNames[i]] != "function")
            if(typeof funcs[i] == "function")
                window[funcNames[i]] = funcs[i];
            else
                throw new ReferenceError(`No "${ funcs[i] }" function exists.`);

    portDisconnectFunc = onDisconnect;

    port = chrome.runtime.connect(null, { name: nameArg });
    port.onMessage.addListener(onPortMessage);
    port.onDisconnect.addListener(onPortDisconnect);
}

function sendToPort(func, ...args) {
    if(port == null)
        return;

    const [ cmd ] = args;

    if(portResponses.hasOwnProperty(cmd))
        throw new Error(`Multiple messages of command ${ cmd } found. One at a time, please.`);

    portResponses[cmd] = func;
    port.postMessage(args);
}

function decouplePort() {
    if(!port || !portDisconnectFunc)
        return;

    port.onMessage.removeListener(onPortMessage);
    port.onDisconnect.removeListener(onPortDisconnect);
}

function onPortMessage(message) {
    const [ cmd, ...data ]  = message,
          isStartupCallback = cmd === null;

    let callback = isStartupCallback ? onStartup : null;

    if(callback === null) {
        callback = portResponses[cmd];

        if(typeof callback == "function")
            delete portResponses[cmd];
        else
            callback = (...args) => onBackgroundMessage(cmd, ...args);
    }

    requestAnimationFrame(() => callback(...data));
}

function onPortDisconnect() {
    port = null;
    portResponses = { };

    if(portDisconnectFunc) {
        portDisconnectFunc();
        portDisconnectFunc = null;
    }
}